==> samplingR_0.1.4.tar.gz: samplingR/R/sys.estimator.R <==
#' @title Parameter estimation on a systematic sample
#'
#' @param N Population size
#' @param sample Vector containing the systematic sample
#' @param estimator Estimator to compute. Can be one of "total", "mean", "proportion", "class total". Default is "total".
#' @param method Method of variance estimation. Can be one of "srs", "strata", "sys".
#' @param alpha Optional value to calculate estimation error and build 1-alpha confidence interval.
#' @param t Number of systematic samples to take with interpenetrating samples method.
#' @param data Population data.
#'
#' @details Variance estimation has no direct formula in systematic sampling, thus estimation method must be done. It should be chosen based on the results of \code{\link{sys.intracorr}}
#' and \code{\link{sys.corrwst}} functions. For sys.intracorr values close to 0 simple random sampling should be used for estimation.
#' For sys.corrwst values close to 0 stratified sampling should be used. If none of them are close to 0 then "sys" estimation method should be used.\cr
#' "sys" method uses interpenetrating samples method in which t systematic samples of size=\eqn{\frac{n}{t}} are taken to estimate. \eqn{\frac{n}{t}} must be even.\cr
#' By choosing the start at random for all the samples they can be considered random taken. With this method population data and t must be given.
#'
#'
#' @return A list containing different interest values:
#'  \itemize{
#'    \item estimator
#'    \item variance
#'    \item sampling.error
#'    \item estimation.error
#'    \item confint
#'  }
#' @export
#'
#' @examples
#' data<-c(1,3,5,2,4,6,2,7,3)
#' sample<-sys.sample(9, 3, data)
#' sys.estimator(N=9, sample, "mean", "srs", 0.05)

sys.estimator<-function(N, sample, estimator=c("total", "mean", "proportion", "class total"), method=c("srs", "strata", "sys"), alpha, data, t){
  estimator<-match.arg(estimator)
  if(estimator != "total" && estimator!="mean" && estimator!="proportion" && estimator!="class total")
    stop('Estimator must be one of c("total", "mean", "proportion", "class total").')
  method=match.arg(method)
  if(method=="sys" && missing(t))stop("t must be provided for interpenetrating samples method estimation")

  n<-length(sample)

  if(method=="srs"){
    return(srs.estimator(N=N, data=sample, estimator=estimator, alpha=alpha))
  }
  else if(method=="strata"){
    #solve odd sample size problem
    if(n%%2!=0){
      sample[n+1]<-sample[srs.sample(n,1)]
      n<-n+1
    }

    k<-N/n
    sample<-data.frame(data=sample, strata=rep(c(1:(n/2)), each=2))
    return(strata.estimator(N=N, Nh=rep(2*k, n/2), data=sample, estimator=estimator, alpha=alpha))
  }
  else{
    if(n%%t!=0)stop("t value does not provide an even sample size")
    #Retrieve list of subsamples
    samples<-list()
    for(i in 1:t){
      aux<-sys.sample(N, n/t, data=data)
      samples[[i]]<-aux
    }
    if(estimator=="total" || estimator=="class total"){
      estimator<-sum(N*sapply(samples, mean))/t
      var<-sum((N*sapply(samples, mean))^2-estimator^2)/(t*(t-1))
      serror<-sqrt(var)
    }
    else if(estimator=="mean" || estimator=="proportion"){
      estimator<-sum(sapply(samples, mean))/t
      var<-sum(sapply(samples, mean)^2-estimator^2)/(t*(t-1))
      serror<-sqrt(var)
    }
    if(!missing(alpha)){
      esterror<-qnorm(1-alpha/2)*serror
      confint<-c(estimator-esterror, estimator+esterror)
      return(list("estimator" = estimator, "variance" = var, "sampling.error" = serror,
                  "estimation.error" = esterror, "confint" = confint))
    }

    return(list("estimator" = estimator, "variance" = var, "sampling.error" = serror))
  }

}

#data<-rnorm(1000,5,2)
#sum(data)
#sample<-sys.sample(1000, 100, data)
#sys.estimator(N=1000, sample, "total", "sys", 0.05, data=data, t=20)


#Ejemplo 1
#Repetir para cada posible muestra
#data<-c(1,3,5,2,4,6,2,7,3)
#sys.anova(data,3)
#sample<-sys.sample(9, 3, data);sample
#sys.intracorr(9,3,data) #0.34375
#sys.corrwst(9,3,data)   #0.4
#sys.estimator(N=9, sample, "mean", "srs", 0.05) #$variance c(1,2,2)=0.07407407 c(3,4,7)=0.962963 c(5,6,3)=0.5185185

#Ejemplo 2
#data<-c(1,3,5,2,4,6,2,7)
#sample<-sys.sample(8, 4, data);sample
#sys.intracorr(8,4,data) #-0.1428571
#sys.corrwst(8,4,data)   #-0.04761905
#sys.estimator(N=8, sample, "mean", "srs", 0.05) #$variance c(1,5,4,2)=0.4166667 c(3,2,6,7)=0.7083333
#sys.estimator(N=8, sample, "mean", "strata", 0.05) #$variance c(1,5,4,2)=0.625 c(3,2,6,7)=0.0625

==> samplingR_0.1.5.tar.gz: samplingR/R/sys.estimator.R <==
#' @title Parameter estimation on a systematic sample
#'
#' @param N Population size
#' @param sample Vector containing the systematic sample
#' @param estimator Estimator to compute. Can be one of "total", "mean", "proportion", "class total". Default is "total".
#' @param method Method of variance estimation. Can be one of "srs", "strata", "sys".
#' @param alpha Optional value to calculate estimation error and build 1-alpha confidence interval.
#' @param t Number of systematic samples to take with interpenetrating samples method.
#' @param data Population data.
#'
#' @details Variance estimation has no direct formula in systematic sampling, thus estimation method must be done. Refer to \code{\link{sys.intracorr}}
#' and \code{\link{sys.intercorr}} functions details for more information. \cr
#' "sys" method uses interpenetrating samples method in which t systematic samples of size=\eqn{\frac{n}{t}} are taken to estimate. \eqn{\frac{n}{t}} must be even.\cr
#' By choosing the start at random for all the samples they can be considered random taken. With this method population data and t must be given.
#'
#'
#' @return A list containing different interest values:
#'  \itemize{
#'    \item estimator
#'    \item variance
#'    \item sampling.error
#'    \item estimation.error
#'    \item confint
#'  }
#' @export
#'
#' @examples
#' data<-c(1,3,5,2,4,6,2,7,3)
#' sample<-sys.sample(9, 3, data)
#' sys.estimator(N=9, sample, "mean", "srs", 0.05)

sys.estimator<-function(N, sample, estimator=c("total", "mean", "proportion", "class total"), method=c("srs", "strata", "sys"), alpha, data, t){
  estimator<-match.arg(estimator)
  if(estimator != "total" && estimator!="mean" && estimator!="proportion" && estimator!="class total")
    stop('Estimator must be one of c("total", "mean", "proportion", "class total").')
  method=match.arg(method)
  if(method=="sys" && missing(t))stop("t must be provided for interpenetrating samples method estimation")

  n<-length(sample)

  if(method=="srs"){
    return(srs.estimator(N=N, data=sample, estimator=estimator, alpha=alpha))
  }
  else if(method=="strata"){
    #solve odd sample size problem
    if(n%%2!=0){
      sample[n+1]<-sample[srs.sample(n,1)]
      n<-n+1
    }

    k<-N/n
    sample<-data.frame(data=sample, strata=rep(c(1:(n/2)), each=2))
    return(strata.estimator(N=N, Nh=rep(2*k, n/2), data=sample, estimator=estimator, alpha=alpha))
  }
  else{
    if(n%%t!=0)stop("t value does not provide an even sample size")
    #Retrieve list of subsamples
    samples<-list()
    for(i in 1:t){
      aux<-sys.sample(N, n/t, data=data)
      samples[[i]]<-aux
    }
    if(estimator=="total" || estimator=="class total"){
      estimator<-sum(N*sapply(samples, mean))/t
      var<-sum((N*sapply(samples, mean))^2-estimator^2)/(t*(t-1))
      serror<-sqrt(var)
    }
    else if(estimator=="mean" || estimator=="proportion"){
      estimator<-sum(sapply(samples, mean))/t
      var<-sum(sapply(samples, mean)^2-estimator^2)/(t*(t-1))
      serror<-sqrt(var)
    }
    if(!missing(alpha)){
      esterror<-qnorm(1-alpha/2)*serror
      confint<-c(estimator-esterror, estimator+esterror)
      return(list("estimator" = estimator, "variance" = var, "sampling.error" = serror,
                  "estimation.error" = esterror, "confint" = confint))
    }

    return(list("estimator" = estimator, "variance" = var, "sampling.error" = serror))
  }

}

#data<-rnorm(1000,5,2)
#sum(data)
#sample<-sys.sample(1000, 100, data)
#sys.estimator(N=1000, sample, "total", "sys", 0.05, data=data, t=20)


#Ejemplo 1
#Repetir para cada posible muestra
#data<-c(1,3,5,2,4,6,2,7,3)
#sys.anova(data,3)
#sample<-sys.sample(9, 3, data);sample
#sys.intracorr(9,3,data) #0.34375
#sys.intercorr(9,3,data)   #0.4
#sys.estimator(N=9, sample, "mean", "srs", 0.05) #$variance c(1,2,2)=0.07407407 c(3,4,7)=0.962963 c(5,6,3)=0.5185185

#Ejemplo 2
#data<-c(1,3,5,2,4,6,2,7)
#sample<-sys.sample(8, 4, data);sample
#sys.intracorr(8,4,data) #-0.1428571
#sys.intercorr(8,4,data)   #-0.04761905
#sys.estimator(N=8, sample, "mean", "srs", 0.05) #$variance c(1,5,4,2)=0.4166667 c(3,2,6,7)=0.7083333
#sys.estimator(N=8, sample, "mean", "strata", 0.05) #$variance c(1,5,4,2)=0.625 c(3,2,6,7)=0.0625
